<?php
/**
*
* @package acp
*
* @author evil3 (2008)
* @author Kazekage (2013)
* @copyright (c) 2013 Kazekage
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

/**
* @package acp
*/
class acp_troll
{
	public $u_action;
	protected $new_config;

	public function main($id, $mode)
	{
		global $user;

		$user->add_lang('mods/info_acp_troll');

		switch ($mode)
		{
			case 'manage':
				$action = request_var('m', '');
				$troll_act = request_var('a', '');
				
				$this->tpl_name = 'acp_troll_actions';
				
				if (!$action)
				{
					$this->display_actions();
				}
				else
				{
					$this->manage_actions($action, $troll_act);
				}
			break;
			case 'config':
				$this->display_config();
			break;
		}
	}

	public function display_config()
	{
		global $user, $template;
		global $config;

		// set new config
		$this->new_config = $config;

		// initialisation
		$error = $display_vars = array();

		// page settings
		$this->page_title = $user->lang['ACP_TROLL_CONFIG'];
		$this->tpl_name = 'acp_board';

		// form key
		$form_key = 'acp_troll_config';

		add_form_key($form_key);

		// request vars
		$cfg_array = (isset($_REQUEST['config'])) ? utf8_normalize_nfc(request_var('config', array('' => ''), true)) : $this->new_config;
		$submit = isset($_POST['submit']) ? true : false;

		// set our display vars
		$display_vars = array(
			'legend1'				=> 'TROLL_GENERAL',
			'troll_virus'			=> array('lang' => 'TROLL_VIRUS',				'validate' => 'bool',	'type' => 'radio:enabled_disabled', 'explain' => true),
			'troll_chance'			=> array('lang' => 'TROLL_CHANCE',				'validate' => 'int:0:100',	'type' => 'text:4:10', 'explain' => true, 'append' => '%'),		
			'troll_group'			=> array('lang' => 'TROLL_GROUP',				'validate' => 'int:0',	'type' => 'text:4:10', 'explain' => true),
			'troll_group_main'			=> array('lang' => 'TROLL_GROUP_MAIN',					'validate' => 'bool',	'type' => 'radio:yes_no', 'explain' => true),
			
			'legend2'				=> 'TROLL_BOARD',
			'troll_post'			=> array('lang' => 'TROLL_POST',				'validate' => 'bool',	'type' => 'radio:enabled_disabled', 'explain' => true),
			'troll_pm'				=> array('lang' => 'TROLL_PM',					'validate' => 'bool',	'type' => 'radio:enabled_disabled', 'explain' => true),
			'troll_email'			=> array('lang' => 'TROLL_EMAIL',				'validate' => 'bool',	'type' => 'radio:enabled_disabled', 'explain' => true),
		);

		// We validate the complete config if whished
		validate_config_vars($display_vars, $cfg_array, $error);

		if ($submit && !check_form_key($form_key))
		{
			$error[] = $user->lang['FORM_INVALID'];
		}

		// Do not write values if there is an error
		if ($submit && sizeof($error))
		{
			trigger_error(implode('<br />', $error) . adm_back_link($this->u_action), E_USER_WARNING);
		}

		// We go through the display_vars to make sure no one is trying to set variables he/she is not allowed to...
		foreach ($display_vars as $config_name => $null)
		{
			if (!isset($cfg_array[$config_name]) || strpos($config_name, 'legend') !== false)
			{
				continue;
			}

			$this->new_config[$config_name] = $config_value = $cfg_array[$config_name];

			if ($submit)
			{
				set_config($config_name, $config_value);
			}
		}

		if ($submit)
		{
			add_log('admin', 'LOG_TROLL_CONFIG');

			trigger_error($user->lang['CONFIG_UPDATED'] . adm_back_link($this->u_action));
		}

		// Output relevant page
		foreach ($display_vars as $config_key => $vars)
		{
			if (!is_array($vars) && strpos($config_key, 'legend') === false)
			{
				continue;
			}

			if (strpos($config_key, 'legend') !== false)
			{
				$template->assign_block_vars('options', array(
					'S_LEGEND'		=> true,
					'LEGEND'		=> (isset($user->lang[$vars])) ? $user->lang[$vars] : $vars,
				));

				continue;
			}

			$type = explode(':', $vars['type']);

			$l_explain = '';
			if ($vars['explain'] && isset($vars['lang_explain']))
			{
				$l_explain = (isset($user->lang[$vars['lang_explain']])) ? $user->lang[$vars['lang_explain']] : $vars['lang_explain'];
			}
			else if ($vars['explain'])
			{
				$l_explain = (isset($user->lang[$vars['lang'] . '_EXPLAIN'])) ? $user->lang[$vars['lang'] . '_EXPLAIN'] : '';
			}

			$content = build_cfg_template($type, $config_key, $this->new_config, $config_key, $vars);

			if (empty($content))
			{
				continue;
			}

			$template->assign_block_vars('options', array(
				'KEY'			=> $config_key,
				'TITLE'			=> (isset($user->lang[$vars['lang']])) ? $user->lang[$vars['lang']] : $vars['lang'],
				'S_EXPLAIN'		=> $vars['explain'],
				'TITLE_EXPLAIN'	=> $l_explain,
				'CONTENT'		=> build_cfg_template($type, $config_key, $this->new_config, $config_key, $vars),

			));

			unset($display_vars[$config_key]);
		}

		// template vars
		$template->assign_vars(array(
			'L_TITLE'				=> $this->page_title,
			'L_TITLE_EXPLAIN'		=> $user->lang['ACP_TROLL_CONFIG_EXPLAIN'],

			'U_ACTION'				=> $this->u_action,
		));
	}

	public function display_actions()
	{
		global  $db, $template, $user, $phpbb_root_path, $phpEx;
		
		$this->page_title = $user->lang['ACP_TROLL_ACT'];
		
		$troll_actions = $troll_actions_db = $troll_actions_file = array();
		$troll_db_only = $troll_file_only = $troll_enabled = array();
		
		$troll_dir = scandir($phpbb_root_path . 'includes/troll/');
		
		$sql = 'SELECT action_name, action_enabled FROM ' . TROLL_ACTIONS_TABLE . ' ORDER BY action_name';
		$result = $db->sql_query($sql);
		
		while ($row = $db->sql_fetchrow($result))
		{
			$troll_actions_db[] = stripslashes($row['action_name']);
			if (!empty($row['action_enabled']))
			{
				$troll_enabled[] = stripslashes($row['action_name']);
			}
		}
		$db->sql_freeresult($result);
		
		foreach ($troll_dir as $file)
		{
			if (preg_match('#^troll_(.+)\.' . preg_quote($phpEx, '#') . '$#', $file))
			{
				$troll_actions_file[] = preg_replace('#^troll_(.+)\.' . preg_quote($phpEx, '#') . '$#', '$1', $file);
			}
		}
		// se hacen las listas de las acciones instaladas, las que solo estan en la bd y las desinstaladas
		$troll_actions = array_intersect($troll_actions_db, $troll_actions_file);
		$troll_db_only = array_diff($troll_actions_db, $troll_actions_file);
		$troll_file_only = array_diff($troll_actions_file, $troll_actions_db);
		
		foreach ($troll_actions as $troll_act)
		{
			if (in_array ($troll_act, $troll_enabled))
			{
				$img  = '<img src="images/icon_folder.gif" alt="' . $user->lang['ACTION_ENABLED'] .'" />';
			}
			else
			{
				$img = '<img src="images/icon_folder_lock.gif" alt="' . $user->lang['DEACTIVATED_ACTION'] .'" />';
			}
			
			$url = $this->u_action . '&amp;a=' . $troll_act;
			
			$template->assign_block_vars('actions', array(
				'NAME'			=> $troll_act,
				'IMG'				=> $img,
				'DESC'			=> (!empty($user->lang[strtoupper($troll_act) . '_DESC']) ? $user->lang[strtoupper($troll_act) . '_DESC'] : ''),
				'U_EDIT'			=> $url . '&amp;m=edit',
				'U_DELETE'			=> $url . '&amp;m=delete',
			));
		}
		
		foreach ($troll_db_only as $troll_act)
		{
			$url = $this->u_action . '&amp;a=' . $troll_act;

			$template->assign_block_vars('db_only', array(
				'NAME'			=> $troll_act,
				'DESC'			=> (!empty($user->lang[strtoupper($troll_act) . '_DESC']) ? $user->lang[strtoupper($troll_act) . '_DESC'] : ''),
				'IMG'				=> '<img src="images/icon_folder_link.gif" alt="' . $user->lang['DEACTIVATED_ACTION'] .'" />',
				'U_DELETE'			=> $url . '&amp;m=delete',
			));
		}
		
		foreach ($troll_file_only as $troll_act)
		{
			$url = $this->u_action . '&amp;a=' . $troll_act;

			if (!preg_match('#^[a-zA-Z][a-zA-Z0-9_]*$#',  $troll_act))
			{
				$desc = $user->lang['INVALID_NAME'];
			}
			else
			{
				$desc = (!empty($user->lang[strtoupper($troll_act) . '_DESC']) ? $user->lang[strtoupper($troll_act) . '_DESC'] : '');
			}

			$template->assign_block_vars('file_only', array(
				'NAME'			=> $troll_act,
				'DESC'			=> $desc,
				'IMG'				=> '<img src="images/icon_folder_link.gif" alt="' . $user->lang['DEACTIVATED_ACTION'] .'" />',
				'U_ADD'			=> $url . '&amp;m=add',
			));
		}
	}
	
	public function manage_actions($action, $troll_act)
	{
		global $db, $template, $user, $phpbb_root_path, $phpEx, $cache;
		
		$this->page_title = $user->lang['ACP_TROLL_ACT'];

		switch ($action)
		{
			case 'add':
			{
				if (!preg_match('#^[a-zA-Z][a-zA-Z0-9_]*$#',  $troll_act))
				{
					trigger_error($user->lang['INVALID_NAME_EXPLAIN'] . adm_back_link($this->u_action), E_USER_WARNING);
				}
				
				$sql = 'SELECT action_name FROM ' . TROLL_ACTIONS_TABLE . ' WHERE action_name = "' . $db->sql_escape($troll_act) .'"';
				$result = $db->sql_query($sql);
				
				if ((bool) $db->sql_fetchfield('action_name'))
				{
					trigger_error($user->lang['ACTION_IN_DB'] . adm_back_link($this->u_action), E_USER_WARNING);
				}
				$db->sql_freeresult($result);
				
				$sql_array = array(
					'action_name'	=> $db->sql_escape($troll_act),
					'action_enabled'		=> 0,
					//  se aade el valor 0 porque siempre debe haber algo en un campo de texto en la bd, 
					// cuando se edite la accin se cambiar por un array serializado
					'action_settings'		=> '0',
				);
				
				$sql = 'INSERT INTO ' . TROLL_ACTIONS_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_array);
				$db->sql_query($sql);
				
				$cache->destroy('_troll_actions');
				add_log('admin', 'LOG_ACTION_ADD', $troll_act);
				trigger_error($user->lang['ACTION_LIST_UPDATED'] . adm_back_link($this->u_action));
			}
			break;
			
			case 'delete':
			{
				if (confirm_box(true))
				{
					$sql = 'DELETE FROM ' . TROLL_ACTIONS_TABLE . ' WHERE action_name = "' . $db->sql_escape($troll_act) .'"';
					$db->sql_query($sql);
					
					$cache->destroy('_troll_actions');
					add_log('admin', 'LOG_ACTION_DELETE', $troll_act);
					trigger_error($user->lang['ACTION_LIST_UPDATED'] . adm_back_link($this->u_action));
				} 
				else
				{
					confirm_box(false, $user->lang['CONFIRM_DELETE'], build_hidden_fields(array(
						'a'	=> $troll_act,
						'm'	=> 'delete',
					)));
				}
				$this->display_actions();
			}
			break;
			
			case 'edit':
			{
				// template
				$this->tpl_name = 'acp_board';
				
				// initialisation
				$error = $config_vars = $cfg_array = array();
				
				if (isset($_REQUEST['config']))
				{
					$cfg_array =  utf8_normalize_nfc(request_var('config', array('' => ''), true));
					$enabled = (bool) $cfg_array['action_enabled'];
				}
				else
				{
					$sql = 'SELECT action_name, action_enabled, action_settings 
						FROM ' . TROLL_ACTIONS_TABLE . '
						WHERE action_name = "' . $db->sql_escape($troll_act) .'"';
					$result = $db->sql_query($sql);
						
					if (!$row = $db->sql_fetchrow($result))
					{
						trigger_error($user->lang['NO_ACTION_IN_DB'] . adm_back_link($this->u_action), E_USER_WARNING);
					}
					$db->sql_freeresult($result);
					
					$cfg_array = (array) @unserialize($row['action_settings']);
					$enabled = (bool) $row['action_enabled'];
					$cfg_array = array_merge(array('action_enabled' => $enabled), $cfg_array);

				}
				
				// form key
				$form_key = 'acp_ta_config';
				add_form_key($form_key);
			
				$submit = isset($_POST['submit']) ? true : false;
				
				// include the action class
				$class_name = "troll_$troll_act";
				
				include($phpbb_root_path . 'includes/troll/base.' . $phpEx);
				include("{$phpbb_root_path}includes/troll/$class_name.$phpEx");
				
				$action = new $class_name();
				
				// set our display vars
				$config_vars = array(
					'legend0'				=> 'GENERAL',
					'action_enabled'			=> array('lang' => 'ACTION_ENABLED',				'validate' => 'bool',	'type' => 'radio:yes_no', 'explain' => true),
				);
				
				$config_vars = array_merge($config_vars, (array) $action->config);
				
				// We validate the complete config if whished
				validate_config_vars($config_vars, $cfg_array, $error);

				if ($submit && !check_form_key($form_key))
				{
					$error[] = $user->lang['FORM_INVALID'];
				}
			
				if ($submit && sizeof($error))
				{
					trigger_error(implode('<br />', $error) . adm_back_link($this->u_action), E_USER_WARNING);
				}
			
				if ($submit)
				{
					array_shift($cfg_array);

					$sql_array = array(
						'action_enabled'		=> $enabled,
						'action_settings'		=> serialize($cfg_array),
					);	
					
					$sql = 'UPDATE ' . TROLL_ACTIONS_TABLE . ' SET ' . $db->sql_build_array('UPDATE', $sql_array) .
						' WHERE action_name = "' . $db->sql_escape($troll_act) .'"';
					$db->sql_query($sql);

					add_log('admin', 'LOG_ACTION_EDIT', $troll_act);

					$cache->destroy('_troll_actions');
					
					trigger_error($user->lang['ACTION_UPDATED'] . adm_back_link($this->u_action));
				}
			
				// Output relevant page
				foreach ($config_vars as $config_key => $vars)
				{
					if (!is_array($vars) && strpos($config_key, 'legend') === false)
					{
						continue;
					}
			
					if (strpos($config_key, 'legend') !== false)
					{
						$template->assign_block_vars('options', array(
							'S_LEGEND'		=> true,
							'LEGEND'		=> (isset($user->lang[$vars])) ? $user->lang[$vars] : $vars,
						));
			
						continue;
					}
			
					$type = explode(':', $vars['type']);
			
					$l_explain = '';
					if ($vars['explain'] && isset($vars['lang_explain']))
					{
						$l_explain = (isset($user->lang[$vars['lang_explain']])) ? $user->lang[$vars['lang_explain']] : $vars['lang_explain'];
					}
					else if ($vars['explain'])
					{
						$l_explain = (isset($user->lang[$vars['lang'] . '_EXPLAIN'])) ? $user->lang[$vars['lang'] . '_EXPLAIN'] : '';
					}
			
					$content = build_cfg_template($type, $config_key, $cfg_array, $config_key, $vars);
			
					if (empty($content))
					{
						continue;
					}
			
					$template->assign_block_vars('options', array(
						'KEY'			=> $config_key,
						'TITLE'			=> (isset($user->lang[$vars['lang']])) ? $user->lang[$vars['lang']] : $vars['lang'],
						'S_EXPLAIN'		=> $vars['explain'],
						'TITLE_EXPLAIN'	=> $l_explain,
						'CONTENT'		=> $content,
					));
			
					unset($config_vars[$config_key]);
				}
			
				$template->assign_vars(array(
					'L_TITLE'				=> $troll_act,
					'L_TITLE_EXPLAIN'		=> (!empty($user->lang[strtoupper($troll_act) . '_DESC']) ? $user->lang[strtoupper($troll_act) . '_DESC'] : ''),

					'U_ACTION'				=> $this->u_action . '&amp;m=edit&amp;a=' . $troll_act,
				));
			}
			break;
			
			default:
				$this->display_actions();
			break;
		}
	}
}

?>