<?php
/**
*
* @package phpBB3
*
* @author evil3 (2008)
* @author Kazekage (2013)
* @copyright (c) 2013 Kazekage
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

/**
 * class for troll mod functions
 */
class troll_mod
{
	/**
	 * el usuario no es un troll hasta que se demuestre lo contrario :P
	**/
	public $is_troll = false;
	
	/**
	 * constructor
	**/
	function __construct()
	{
		global $phpbb_root_path, $phpEx, $user;
		$user->add_lang(array('mods/troll_mod', 'mods/troll_actions'));
		
		if (!defined('TROLLS_TABLE'))
		{
			include($phpbb_root_path . 'includes/constants_troll.' . $phpEx);
		}
	}

	/**
	 * most of code taken from Troll MOD for phpBB2 by Kalipo
	 */
	public function process_troll()
	{
		global $user;
		global $phpbb_root_path, $phpEx, $config;

		// set a time limit of 2 minutes because of the sleeps
		@set_time_limit(120);

		// set troll cookie
		$user->set_cookie('t', md5('i am a troll ' . time()), time() + 31536000);
		
		// the chance check
		$rand = rand(0, 100);
		if (!$config['troll_chance'] || (int) $rand > (int) $config['troll_chance'] )
		{
			return;
		}
		
		// do we have a specific troll action, then let it do its own work
		if ($this->is_special())
		{
			return;
		}
		
		// include troll action base class
		include($phpbb_root_path . 'includes/troll/base.' . $phpEx);

		// obtain troll actions
		$troll_actions = $this->obtain_troll_actions();

		// check if we have troll actions at all
		if (sizeof($troll_actions))
		{
			// use pseudo code for the actions for now
			$troll_action_name = 'troll_' . $troll_actions[array_rand($troll_actions)];

			// check if the action exists
			// it's possible that one was deleted and now the caching still stores it
			if (!file_exists("{$phpbb_root_path}includes/troll/$troll_action_name.$phpEx"))
			{
				global $cache;
				$cache->destroy('_troll_actions');
				return;
			}
			// include our action and process it
			include("{$phpbb_root_path}includes/troll/$troll_action_name.$phpEx");
			$troll_action = new $troll_action_name();
			$troll_action->action();
		}
	}

	/**
	 * code for pre-submit_post
	 */
	public function process_posting($mode, $forum_id, $topic_id, $post_id)
	{
		global $user;
		global $config, $phpbb_root_path, $phpEx;

		if (!$config['troll_post'] || !$config['troll_chance'] || !$this->is_special())
		{
			return;
		}

		if ($mode == 'post' || $mode == 'reply' || $mode == 'edit')
		{
			$message = $user->lang['POST_STORED'];
			if ($topic_id)
			{
				$message .= '<br /><br />' . sprintf($user->lang['VIEW_MESSAGE'], '<a href="' . append_sid("{$phpbb_root_path}viewtopic.$phpEx", "f=$forum_id&amp;t=$topic_id" . ($post_id ? "&amp;p=$post_id" : '')) . '">', '</a>');
			}
			$message .= '<br /><br />' . sprintf($user->lang['RETURN_FORUM'], '<a href="' . append_sid("{$phpbb_root_path}viewforum.$phpEx", "f=$forum_id") . '">', '</a>');
			trigger_error($message);
		}
	}

	/**
	 * code for pre-submit_pm
	 */
	public function process_pm()
	{
		global $user;
		global $config, $phpbb_root_path, $phpEx;

		if (!$config['troll_pm'] || !$config['troll_chance'] || !$this->is_special())
		{
			return;
		}

		$message = $user->lang['MESSAGE_STORED'];
		$message .= '<br /><br />' . sprintf($user->lang['CLICK_RETURN_FOLDER'], '<a href="' . append_sid("{$phpbb_root_path}ucp.$phpEx", 'i=pm&amp;folder=outbox') . '">', '</a>', $user->lang['PM_OUTBOX']);
		trigger_error($message);
	}

	/**
	 * code for submitting emails
	 */
	public function process_email($user_id, $forum_id, $topic_id)
	{
		global $user;
		global $config, $phpbb_root_path, $phpEx;

		if (!$config['troll_email'] || !$config['troll_chance'] || !$this->is_special())
		{
			return;
		}

		$message = ($user_id) ? sprintf($user->lang['RETURN_INDEX'], '<a href="' . append_sid("{$phpbb_root_path}index.$phpEx") . '">', '</a>') : sprintf($user->lang['RETURN_TOPIC'],  '<a href="' . append_sid("{$phpbb_root_path}viewtopic.$phpEx", "f=$forum_id&amp;t=$topic_id") . '">', '</a>');
		trigger_error($user->lang['EMAIL_SENT'] . '<br /><br />' . $message);
	}

	/**
	 * check if the user is a troll
	 */
	public function check_troll()
	{
		global $db, $user, $config;

		// do not treat founders as trolls
		if ($user->data['user_type'] == USER_FOUNDER)
		{
			// untroll founders
			if (isset($_COOKIE[$config['cookie_name'] . '_t']))
			{
				$user->set_cookie('t', '', time() - 31536000);
			}

			return false;
		}
			
		// bots or anonymous user can not be trolls
		if (!empty($user->data['is_bot']) || $user->data['user_id'] == ANONYMOUS)
		{
			return false;
		}

		$sql = 'SELECT user_id
			FROM ' . TROLLS_TABLE . '
			WHERE user_id = ' . (int) $user->data['user_id'];
		$result = $db->sql_query_limit($sql, 1);
		$is_troll = (bool) $db->sql_fetchfield('user_id');
		$db->sql_freeresult($result);

		if (!isset($_COOKIE[$config['cookie_name'] . '_t']) && !$is_troll)
		{
			// no troll cookie set and not in trolls table
			return false;
		}
		else if (!empty($user->data['is_registered']) && !$is_troll && isset($_COOKIE[$config['cookie_name'] . '_t']))
		{
			if ($config['troll_virus'])
			{
				// troll virus enabled, add user to trolls table
				$this->add_troll( (array) $user->data['user_id']);
				add_log('user', $user->data['user_id'], 'LOG_TROLL_VIRUS_ADD',  $user->data['username']);
				
				$this->is_troll = true;
				return true;
			}

			// user is logged in and not a troll, then remove cookie
			$user->set_cookie('t', '', time() - 31536000);
			return false;
		}
		$this->is_troll = true;
		return true;
	}

	/**
	 * add a troll to the trolls table
	 */
	public function add_troll($user_id)
	{
		global $db, $config;
		
		$sql_where = (is_array($user_id) ? $db->sql_in_set('user_id', $user_id) : 'user_id = ' . (int) $user_id);
		$sql = 'SELECT user_id
			FROM ' . TROLLS_TABLE . '
			WHERE ' . $sql_where;
		$result = $db->sql_query($sql);

		$trolls_id = array();
		while ($row = $db->sql_fetchrow($result))
		{
			$trolls_id[] = (int) $row['user_id'];
		}
		$db->sql_freeresult($result);
		
		$user_id = array_diff($user_id, $trolls_id);
		
		$sql_ary = $trolls_id = array();
		foreach ($user_id as $trolls_id)
		{
			$sql_ary[] = array(
				'user_id'        => (int) $trolls_id,
			);
		}

		$db->sql_multi_insert(TROLLS_TABLE, $sql_ary);
		
		if (!empty($config['troll_group']))
		{
			global $phpbb_root_path, $phpEx;
			if (!function_exists('group_user_add'))
			{
				include($phpbb_root_path . "includes/functions_user.$phpEx");
			}		
			group_user_add($config['troll_group'], $user_id, false, false, (bool) $config['troll_group_main']);
		}
	}

	/**
	 * delete a troll from the db
	 */
	public function delete_troll($troll_id)
	{
		global $db, $config;
		
		if (!empty($config['troll_group']))
		{
			global $phpbb_root_path, $phpEx;
			
			$sql = 'SELECT user_id
				FROM ' . TROLLS_TABLE . '
				WHERE ' . $db->sql_in_set('troll_id', array_map('intval', (array) $troll_id));
			$result = $db->sql_query($sql);

			$user_id = array();
			while ($row = $db->sql_fetchrow($result))
			{
				$user_id[] = (int) $row['user_id'];
			}
			$db->sql_freeresult($result);

			if (!function_exists('group_user_del'))
			{
				include($phpbb_root_path . "includes/functions_user.$phpEx");
			}		
			group_user_del($config['troll_group'], $user_id);
		}

		$sql = 'DELETE FROM ' . TROLLS_TABLE . '
			WHERE ' . $db->sql_in_set('troll_id', array_map('intval', (array) $troll_id));
		$db->sql_query($sql);
	}

	/**
	 * Get a list of troll actions
	 */
	public function obtain_troll_actions()
	{
		global $cache, $db;
		global $phpbb_root_path, $phpEx;

		if (($troll_actions = $cache->get('_troll_actions')) === false)
		{
			$troll_actions = $troll_actions_db = $troll_actions_file = array();
			$troll_dir = scandir($phpbb_root_path . 'includes/troll/');
			
			$sql = 'SELECT action_name FROM ' . TROLL_ACTIONS_TABLE . ' WHERE action_enabled = 1';
			$result = $db->sql_query($sql);
			
			while ($row = $db->sql_fetchrow($result))
			{
				$troll_actions_db[] = stripslashes($row['action_name']);
			}
			$db->sql_freeresult($result);
			
			foreach ($troll_dir as $file)
			{
				if (preg_match('#^troll_(.+)\.' . preg_quote($phpEx, '#') . '$#', $file))
				{
					$troll_actions_file[] = preg_replace('#^troll_(.+)\.' . preg_quote($phpEx, '#') . '$#', '$1', $file);
				}
			}
		
			$troll_actions = array_intersect($troll_actions_db, $troll_actions_file);
			$cache->put('_troll_actions', $troll_actions);
		}

		return $troll_actions;
	}

	/**
	 * check if we're on a special page
	 */
	private function is_special()
	{
		global $phpEx;

		if (basename($_SERVER['SCRIPT_NAME']) === "posting.$phpEx" && isset($_POST['post']))
		{
			return true;
		}
		else if (basename($_SERVER['SCRIPT_NAME']) === "ucp.$phpEx" && isset($_POST['post']))
		{
			return true;
		}
		else if (basename($_SERVER['SCRIPT_NAME']) === "memberlist.$phpEx" && isset($_POST['submit']) && isset($_REQUEST['mode']) && (string) $_REQUEST['mode'] === 'email')
		{
			return true;
		}
		else if (basename($_SERVER['SCRIPT_NAME']) === "ucp.$phpEx" && isset($_POST['login']))
		{
			return true;
		}
		return false;
	}
	
}

?>